@@ -52,7 +52,8 @@ def iterated_date(He, t_guess,
     the He age equation is not well-behaved (e.g., a radionuclide value
     is negative). In this case, a NaN value is returned.
     
-    Note that at least one radionuclide value must be specified.
+    Note that at least one radionuclide value must be specified. If all
+    nuclides are set to 0, nan is returned
     
     Parameters
     ----------
@@ -98,20 +99,31 @@ def iterated_date(He, t_guess,
     # based on a constant ratio to U238
     if U235 is None:
         U235 = U238/137.818
-    try:
-        # First convert ints or floats to arrays of len(1) 
-        # to accomodate array datatype
-        single_date = False
+    # If no radionuclides were passed, return all nans
+    if np.all(U238 == 0) and np.all(U235 == 0) and np.all(Th232 == 0) and np.all(Sm147 == 0):
         if type(t_guess) != np.ndarray:
-            single_date = True
-            t_guess = np.array([t_guess])
-        # create list of current and previous time (with initial
-        # value of 0) to track amount of change between iterations
-        times = [np.zeros(len(t_guess)), np.zeros(len(t_guess)), t_guess]
-        n = 0
-        ignores = []
-        # restricting to n<100 is really just a safeguard. In practice, most iterated
-        # dates should require <10 cycles to calculate
+            return np.nan
+        else:
+            nan_return = np.empty(len(He))
+            nan_return.fill(np.nan)
+            return nan_return
+    # First convert ints or floats to arrays of len(1) 
+    # to accomodate array datatype
+    single_date = False
+    if type(t_guess) != np.ndarray:
+        single_date = True
+        t_guess = np.array([t_guess])
+    # create list of current and previous time (with initial
+    # value of 0) to track amount of change between iterations
+    # print(t_guess)
+    times = [np.zeros(len(t_guess)), np.zeros(len(t_guess)), t_guess]
+    n = 0
+    ignores = []
+    # restricting to n<100 is really just a safeguard. In practice, most iterated
+    # dates should require <10 cycles to calculate
+    # suppress NaN warnings because NaNs are tracked independently and removed.
+    # Allowing NaNs results in much better performance
+    with np.errstate(divide='ignore', invalid='ignore', over='ignore'):
         while np.nanmean(np.delete(abs(times[2]-times[1]),ignores)) > 1 and n<100:
             n+=1
             times[0] = times[1]
@@ -129,27 +141,32 @@ def iterated_date(He, t_guess,
             # give the method three cycles to converge, and then stop evaluating
             # samples that aren't converging, defined as reversal of convergence
             # direction after three cycles, constrained to changes of >1 year to
-            # avoid floating point errors
+            # avoid floating point errors. The with statement here prevents a divide
+            # by zero warning, which should arise if the code is working properly
+            # (i.e., when there is no change along the array, many elements will be subtracted
+            # from themselves and divided)
             if n>3:
+                # print(times)
                 nonconv = (times[1]-times[2])/(times[0]-times[1])
                 new_times = times[2]-times[1]
                 new_ignores = np.where((nonconv<0) & (abs(new_times)>1))[0]
                 for i in new_ignores:
                     if i not in ignores:
                         ignores.append(i)
-                
-        final_date = np.delete(times[2], ignores)
-        final_date = final_date[~np.isnan(final_date)]
-        final_date = final_date[~np.isinf(final_date)]
-        # If no dates converged, return NaN
-        if len(final_date) == 0:
-            return np.nan
-        # Convert back to float if array was not passed originally
-        if single_date:
-            final_date = float(final_date[0])
-        return final_date
-    except:
+    
+    # remove non-converged, NaN, and INF values
+    final_date = np.delete(times[2], ignores)
+    final_date = final_date[~np.isnan(final_date)]
+    final_date = final_date[~np.isinf(final_date)]
+    
+    # If no dates converged, return NaN
+    if len(final_date) == 0:
         return np.nan
+    # Convert back to float if array was not passed originally
+    if single_date:
+        final_date = float(final_date[0])
+    # print(final_date)
+    return final_date
 
 def meesters_dunai(He,
                    U238=0, U235=None, Th232=0, Sm147=0,
@@ -168,6 +185,9 @@ def meesters_dunai(He,
     If non-physical values are passed to this function (e.g., negative
     production rates of He), a linearized estimate of date is returned
     instead using the equation t = He/TotalProduction.
+    
+    Note that at least one radionuclide value must be specified. If all
+    nuclides are set to 0, nan is returned
         
     Parameters
     ----------
@@ -212,6 +232,13 @@ def meesters_dunai(He,
     '''
     if U235 is None:
         U235 = U238/137.818
+    if np.all(U238 == 0) and np.all(U235 == 0) and np.all(Th232 == 0) and np.all(Sm147 == 0):
+        if type(He) != np.ndarray:
+            return np.nan
+        else:
+            nan_return = np.empty(len(He))
+            nan_return.fill(np.nan)
+            return nan_return
     # Calculate individual production values
     p238 = 8*l_U238*U238*Ft238
     p235 = 7*l_U235*U235*Ft235
@@ -221,12 +248,14 @@ def meesters_dunai(He,
     p_tot = [p238, p235, p232, p147]
     P = p238 + p235 + p232 + p147
     l_wm = sum([l_tot[i]*p_tot[i]/P for i in range(len(p_tot))])
-    try:
-        return 1/l_wm*np.log((l_wm/P)*He+1)
-    # If the Meesters and Dunai method fails, use
-    # a linearized version of the age equaiton
-    except ValueError:
-        return He/sum(p_tot)
+    # Get the Meesters and Dunai estimate
+    # With highly uncertain nuclide values, negative
+    # log values will sometimes arise. Suppress the
+    # warning output and allow the NaN values to carry
+    # through and be removed in the final steps
+    with np.errstate(invalid='ignore'):
+        MD_date = 1/l_wm*np.log((l_wm/P)*He+1)
+    return MD_date
         
 # Function to get age given He, radioisotopes, and Fts
 def get_date(He,
@@ -280,6 +309,13 @@ def get_date(He,
     '''
     if U235 is None:
         U235 = U238/137.818
+    if np.all(U238 == 0) and np.all(U235 == 0) and np.all(Th232 == 0) and np.all(Sm147 == 0):
+        if type(He) != np.ndarray:
+            nan_return = np.nan
+        else:
+            nan_return = np.empty(len(He))
+            nan_return.fill(np.nan)
+        return {'raw date': nan_return, 'corrected date': nan_return}
     # Use M&D 2005 age as first guess for iterative age
     # start by calculating raw age using 1 for all Ft
     t_app_raw = meesters_dunai(He, U238, U235, Th232, Sm147)
