@@ -23,7 +23,6 @@ histograms and parameterize those histograms if a user so desires.
 """
 
 from scipy.stats import skewnorm as sk
-from scipy.optimize import curve_fit
 import numpy as np
 from .date_calculation import get_date
 
@@ -59,20 +58,20 @@ def make_histogram(mc, parameterize):
     
     # Get stats for skewnormal function if the user called for parameterization
     if parameterize:
-        mean = np.mean(mc)
-        med = np.median(mc)
-        s = np.std(mc)
-        skewness = 3*(mean-med)/s
-        a = skewness*10
-        try:
-            p0 = [a, mean, s]
-            c, cov = curve_fit(sk.pdf, histo[0], histo[1], p0)
-        except:
-            print('parameterization unsuccessful, returning only histogram')
-            return {'histogram': histo,
-                    'fit distribution': [None, None, None]}
+        # Subsample Monte Carlo results if lots of simulations were run.
+        # skewnorm.fit is very slow with large samples, and the accuracy of
+        # the fit is perfectly acceptable with 1e5 samples
+        if len(mc) > int(1e5):
+            sample = np.random.choice(mc, int(1e5), replace = False)
+        else:
+            sample = mc
+        a, u, s = sk.fit(sample)
+        # except:
+        #     print('parameterization unsuccessful, returning only histogram')
+        #     return {'histogram': histo,
+        #             'fit distribution': [None, None, None]}
         return {'histogram': histo,
-                'fit distribution': [c[0],c[1],c[2]]}
+                'fit distribution': [a, u, s]}
     return {'histogram': histo}
 
 def monte_carlo(mc_number, He, He_s=0,
@@ -80,6 +79,10 @@ def monte_carlo(mc_number, He, He_s=0,
                 Ft238=1, Ft235=1, Ft232=1, Ft147=1,
                 U238_s=0, U235_s=None, Th232_s=0, Sm147_s=0,
                 Ft238_s=0, Ft235_s=0, Ft232_s=0, Ft147_s=0,
+                U238_U235_v=0, U238_Th232_v=0, U238_Sm147_v=0,
+                U235_Th232_v=0, U235_Sm147_v=0, Th232_Sm147_v=0,
+                Ft238_Ft235_v=0, Ft238_Ft232_v=0, Ft238_Ft147_v=0,
+                Ft235_Ft232_v=0, Ft235_Ft147_v=0, Ft232_Ft147_v=0,
                 histogram=False, parameterize=False):
     '''
     Performs Monte Carlo modeling for a (U-Th)/He dataset. Note that
@@ -147,6 +150,42 @@ def monte_carlo(mc_number, He, He_s=0,
     Ft147_s : float, optional
         The uncertainty in Ft147
         
+    U238_U235_v : float, optional
+        The covariance between U238 and U235
+    
+    U238_Th232_v : float, optional
+        The covariance between U238 and Th232
+    
+    U238_Sm147_v : float, optional
+        The covariance between U238 and Sm147
+    
+    U235_Th232_v : float, optional
+        The covariance between U235 and Th232
+    
+    U235_Sm147_v : float, optional
+        The covariance between U235 and Sm147
+    
+    Th232_Sm147_v : float, optional
+        The covariance between Th232 and Sm147
+    
+    Ft238_Ft235_v : float, optional
+        The covariance between Ft238 and Ft235
+    
+    Ft238_Ft232_v : float, optional
+        The covariance between Ft238 and Ft232
+    
+    Ft238_Ft147_v : float, optional
+        The covariance between Ft238 and Ft147
+    
+    Ft235_Ft232_v : float, optional
+        The covariance between Ft235 and Ft232
+    
+    Ft235_Ft147_v : float, optional
+        The covariance between Ft235 and Ft147
+    
+    Ft232_Ft147_v : float, optional
+        The covariance between Ft232 and Ft147
+        
     histogram : bool, optional
         Whether to generate a histogram for the Monte Carlo results
         
@@ -157,28 +196,51 @@ def monte_carlo(mc_number, He, He_s=0,
     Returns
     -------
     Produces a dictionary with entries for the raw and alpha-ejection corrected
-    results. Each entry is a dictionary with mean, +/- 68% CI, and % skew. If
-    histogram is true, the histogram for each is returned as a list of the bin
-    centers and a list of the number of samples within the bin. The parameters
-    are associated with the key "fit distribution", and are a list of the three
-    parameters (skewness, location, scale) to calculate a skew-normal distribution
+    results. Each entry is a dictionary with mean, +/- 68% CI, and the number of
+    successful cycles. If histogram is true, the histogram for each is returned 
+    as a list of the bin centers and a list of the number of samples within the 
+    bin. The parameters are associated with the key "fit distribution", and are 
+    a list of the three parameters (skewness, location, scale) to calculate a 
+    skew-normal distribution
     '''
-    # Generate arrays of each datum using its uncertainty
-    # This is strictly gaussian for now, but could include
-    # additional distributions or even arbitrary distributions
-    # in the future.
+    # Helium is always univariate, so generate normal sample here
     He = np.random.normal(He, He_s, mc_number)
-    U238 = np.random.normal(U238, U238_s, mc_number)
+    
+    # Generate radionuclides covariance matrix (3x3 in most cases, 4x4 for U235 measured)
+    # and corresponding multivariate normal arrays
     if U235 == None:
+        rad_means = [U238, Th232, Sm147]
+        rad_cov = [[U238_s**2, U238_Th232_v, U238_Sm147_v],
+                   [U238_Th232_v, Th232_s**2, Th232_Sm147_v],
+                   [U238_Sm147_v, Th232_Sm147_v, Sm147_s**2]]
+        rads = np.random.multivariate_normal(rad_means, rad_cov, mc_number).T
+        U238 = rads[0]
+        Th232 = rads[1]
+        Sm147 = rads[2]
         U235 = U238/137.818
     else:
-        U235 = np.random.normal(U235, U235_s, mc_number)
-    Th232 = np.random.normal(Th232, Th232_s, mc_number)
-    Sm147 = np.random.normal(Sm147, Sm147_s, mc_number)
-    Ft238 = np.random.normal(Ft238, Ft238_s, mc_number)
-    Ft235 = np.random.normal(Ft235, Ft235_s, mc_number)
-    Ft232 = np.random.normal(Ft232, Ft232_s, mc_number)
-    Ft147 = np.random.normal(Ft147, Ft147_s, mc_number)
+        rad_means = [U238, U235, Th232, Sm147]
+        rad_cov = [[U238_s**2, U238_U235_v, U238_Th232_v, U238_Sm147_v],
+                   [U238_U235_v, U235_s**2, U235_Th232_v, U235_Sm147_v],
+                   [U238_Th232_v, U235_Th232_v, Th232_s**2, Th232_Sm147_v],
+                   [U238_Sm147_v, U235_Sm147_v, Th232_Sm147_v, Sm147_s**2]]
+        rads = np.random.multivariate_normal(rad_means, rad_cov, mc_number).T
+        U238 = rads[0]
+        U235 = rads[1]
+        Th232 = rads[2]
+        Sm147 = rads[3]
+    
+    # Generate Ft covariance matrix and multivariate normal arrays
+    Ft_means = [Ft238, Ft235, Ft232, Ft147]
+    Ft_covs = [[Ft238_s**2, Ft238_Ft235_v, Ft238_Ft232_v, Ft238_Ft147_v],
+               [Ft238_Ft235_v, Ft235_s**2, Ft235_Ft232_v, Ft235_Ft147_v],
+               [Ft238_Ft232_v, Ft235_Ft232_v, Ft232_s**2, Ft232_Ft147_v],
+               [Ft238_Ft147_v, Ft235_Ft147_v,Ft232_Ft147_v, Ft147_s**2]]
+    Fts = np.random.multivariate_normal(Ft_means, Ft_covs, mc_number).T
+    Ft238 = Fts[0]
+    Ft235 = Fts[1]
+    Ft232 = Fts[2]
+    Ft147 = Fts[3]
     
     # Call the get_date function to calculate a date for each
     # of the randomly generated data
@@ -191,13 +253,17 @@ def monte_carlo(mc_number, He, He_s=0,
     # Produce statistics and save
     for t in ['raw date', 'corrected date']:
         mean = np.mean(MonteCarlo_t[t])
-        CIs = np.percentile(MonteCarlo_t[t], [15.865, 84.135])
-        # CI_low = mean-CIs[0]
-        # CI_high = CIs[1]-mean
+        CIs_68 = np.percentile(MonteCarlo_t[t], [15.865, 84.135])
+        CIs_95 = np.percentile(MonteCarlo_t[t], [15.865, 84.135])
         mc_results[t]['mean'] = mean
-        mc_results[t]['+68% CI'] = CIs[1]
-        mc_results[t]['-68% CI'] = CIs[0]
-        # mc_results[t]['% Skew'] = (((CI_high/CI_low)-1)*100)
+        mc_results[t]['+68% CI'] = CIs_68[1]
+        mc_results[t]['-68% CI'] = CIs_68[0]
+        mc_results[t]['+95% CI'] = CIs_95[1]
+        mc_results[t]['-95% CI'] = CIs_95[0]
+        # Report how many of the Monte Carlo simulations converged to
+        # an age. For highly uncertain input data, this may be significantly
+        # lower than the number of requested cycles.
+        mc_results[t]['cycles'] = len(MonteCarlo_t[t])
         if histogram:
             histogram_out = make_histogram(MonteCarlo_t[t], parameterize)
             mc_results[t]['histogram'] = histogram_out['histogram']
